@@ -1,27 +1,11 @@
 package Gimp::Fu;
 
-use strict 'vars';
 use Carp;
-use vars qw($VERSION @ISA @EXPORT @EXPORT_OK @EXPORT_FAIL %EXPORT_TAGS
-            @scripts @_params $run_mode %pf_type2string @image_params);
-use Gimp qw(:param);
+use Gimp ();
 use Gimp::Data;
-use File::Basename;
-use base qw(Exporter);
 
 require Exporter;
 
-eval {
-   require Data::Dumper;
-   import Data::Dumper 'Dumper';
-};
-if ($@) {
-   *Dumper = sub {
-      "()";
-   };
-}
-
-
 =cut
 
 =head1 NAME
@@ -70,28 +54,29 @@ module after the C<Gimp> module.
 
 =cut
 
-sub PF_INT8	() { PARAM_INT8		};
-sub PF_INT16	() { PARAM_INT16	};
-sub PF_INT32	() { PARAM_INT32	};
-sub PF_FLOAT	() { PARAM_FLOAT	};
-sub PF_STRING	() { PARAM_STRING	};
-sub PF_COLOR	() { PARAM_COLOR	};
-sub PF_COLOUR	() { PARAM_COLOR	};
-sub PF_IMAGE	() { PARAM_IMAGE	};
-sub PF_LAYER	() { PARAM_LAYER	};
-sub PF_CHANNEL	() { PARAM_CHANNEL	};
-sub PF_DRAWABLE	() { PARAM_DRAWABLE	};
+sub PF_INT8	() { Gimp::PARAM_INT8	};
+sub PF_INT16	() { Gimp::PARAM_INT16	};
+sub PF_INT32	() { Gimp::PARAM_INT32	};
+sub PF_FLOAT	() { Gimp::PARAM_FLOAT	};
+sub PF_STRING	() { Gimp::PARAM_STRING	};
+sub PF_COLOR	() { Gimp::PARAM_COLOR	};
+sub PF_COLOUR	() { Gimp::PARAM_COLOR	};
+sub PF_IMAGE	() { Gimp::PARAM_IMAGE	};
+sub PF_LAYER	() { Gimp::PARAM_LAYER	};
+sub PF_CHANNEL	() { Gimp::PARAM_CHANNEL};
+sub PF_DRAWABLE	() { Gimp::PARAM_DRAWABLE};
 
-sub PF_TOGGLE	() { PARAM_END+1	};
-sub PF_SLIDER	() { PARAM_END+2	};
-sub PF_FONT	() { PARAM_END+3	};
-sub PF_SPINNER	() { PARAM_END+4	};
-sub PF_ADJUSTMENT(){ PARAM_END+5	}; # compatibility fix for script-fu _ONLY_
-sub PF_BRUSH	() { PARAM_END+6	};
-sub PF_PATTERN	() { PARAM_END+7	};
-sub PF_GRADIENT	() { PARAM_END+8	};
-sub PF_RADIO	() { PARAM_END+9	};
-sub PF_CUSTOM	() { PARAM_END+10	};
+sub PF_TOGGLE	() { Gimp::PARAM_END+1	};
+sub PF_SLIDER	() { Gimp::PARAM_END+2	};
+sub PF_FONT	() { Gimp::PARAM_END+3	};
+sub PF_SPINNER	() { Gimp::PARAM_END+4	};
+sub PF_ADJUSTMENT(){ Gimp::PARAM_END+5	}; # compatibility fix for script-fu _ONLY_
+sub PF_BRUSH	() { Gimp::PARAM_END+6	};
+sub PF_PATTERN	() { Gimp::PARAM_END+7	};
+sub PF_GRADIENT	() { Gimp::PARAM_END+8	};
+sub PF_RADIO	() { Gimp::PARAM_END+9	};
+sub PF_CUSTOM	() { Gimp::PARAM_END+10	};
+sub PF_FILE	() { Gimp::PARAM_END+11	};
 
 sub PF_BOOL	() { PF_TOGGLE		};
 sub PF_INT	() { PF_INT32		};
@@ -116,26 +101,32 @@ sub Gimp::RUN_FULLINTERACTIVE (){ Gimp::RUN_INTERACTIVE+100 };	# you don't want
          &PF_ADJUSTMENT	=> 'integer',
          &PF_RADIO	=> 'string',
          &PF_CUSTOM	=> 'string',
+         &PF_FILE	=> 'string',
          &PF_IMAGE	=> 'NYI',
          &PF_LAYER	=> 'NYI',
          &PF_CHANNEL	=> 'NYI',
          &PF_DRAWABLE	=> 'NYI',
 );
 
-@_params=qw(PF_INT8 PF_INT16 PF_INT32 PF_FLOAT PF_VALUE
-            PF_STRING PF_COLOR PF_COLOUR PF_TOGGLE PF_IMAGE
-            PF_DRAWABLE PF_FONT PF_LAYER PF_CHANNEL PF_BOOL
-            PF_SLIDER PF_INT PF_SPINNER PF_ADJUSTMENT
-            PF_BRUSH PF_PATTERN PF_GRADIENT PF_RADIO PF_CUSTOM);
+@_params=qw(PF_INT8 PF_INT16 PF_INT32 PF_FLOAT PF_VALUE PF_STRING PF_COLOR
+            PF_COLOUR PF_TOGGLE PF_IMAGE PF_DRAWABLE PF_FONT PF_LAYER
+            PF_CHANNEL PF_BOOL PF_SLIDER PF_INT PF_SPINNER PF_ADJUSTMENT
+            PF_BRUSH PF_PATTERN PF_GRADIENT PF_RADIO PF_CUSTOM PF_FILE);
 
-@EXPORT = (qw(register main),@_params);
-@EXPORT_OK = qw(interact $run_mode save_image);
-%EXPORT_TAGS = (params => [@_params]);
+#@EXPORT_OK = qw(interact $run_mode save_image);
 
 sub import {
    local $^W=0;
-   shift @_ if $_[0] =~ /::/;
-   Gimp::Fu->export_to_level(1,@_);
+   my $up = caller;
+   shift;
+   @_ = (qw(register main),@_params) unless @_;
+   for (@_) {
+      if ($_ eq ":params") {
+         push (@_, @_params);
+      } else {
+         *{"${up}::$_"} = \&$_;
+      }
+   }
 }
 
 # the old value of the trace flag
@@ -167,6 +158,7 @@ sub help_window(\$$$) {
 
       $b = new Gtk::Text;
       $b->set_editable (0);
+      $b->set_word_wrap (1);
 
       $font = load Gtk::Gdk::Font "9x15bold";
       $font = fontset_load Gtk::Gdk::Font "-*-courier-medium-r-normal--*-120-*-*-*-*-*" unless $font;
@@ -196,7 +188,7 @@ sub help_window(\$$$) {
    $$helpwin->show_all();
 }
 
-sub interact($$$@) {
+sub interact($$$$@) {
    local $^W=0;
    my($function)=shift;
    my($blurb)=shift;
@@ -213,9 +205,15 @@ sub interact($$$@) {
       require Gtk; import Gtk;
       init Gtk; # gross hack...
    };
+   
    if ($@) {
-      Gimp::logger(message => 'the gtk perl module is required to run in interactive mode', function => $function);
-      die "The Gtk perl module is required to run this function ($function) in interactive mode!\n";
+      my @res = map {
+         die "the gtk perl module is required to run\nthis plug-in in interactive mode\n" unless defined $_->[3];
+         $_->[3];
+      } @types;
+      Gimp::logger(message => "the gtk perl module is required to open a dialog\nwindow, running with default values",
+                   fatal => 1, function => $function);
+      return (1,@res);
    }
 
    parse Gtk::Rc Gimp->gtkrc;
@@ -228,7 +226,7 @@ sub interact($$$@) {
      my $t = new Gtk::Tooltips;
      my $w = new Gtk::Dialog;
 
-     set_title $w $0;
+     set_title $w $Gimp::function;
      
      my $h = new Gtk::HBox 0,2;
      $h->add(new Gtk::Label Gimp::wrap_text($blurb,40));
@@ -247,11 +245,12 @@ sub interact($$$@) {
         my($value)=shift;
         
         local *new_PF_STRING = sub {
-           $a=new Gtk::Entry;
-           set_usize $a 0,25;
-           push(@setvals,sub{set_text $a defined $_[0] ? $_[0] : ""});
-           #select_region $a 0,1;
-           push(@getvals,sub{get_text $a});
+           my $e = new Gtk::Entry;
+           set_usize $e 0,25;
+           push(@setvals,sub{set_text $e defined $_[0] ? $_[0] : ""});
+           #select_region $e 0,1;
+           push(@getvals,sub{get_text $e});
+           $a=$e;
         };
 
         if($type == PF_ADJUSTMENT) { # support for scm2perl
@@ -283,7 +282,7 @@ sub interact($$$@) {
               my $setval = sub {
                  $val=$_[0];
                  unless (defined $val && $fs->set_font_name ($val)) {
-                    warn "illegal default font description: $val" if defined $val;
+                    warn "illegal default font description for $function: $val\n" if defined $val;
                     $val=$def;
                     $fs->set_font_name ($val);
                  }
@@ -351,8 +350,9 @@ sub interact($$$@) {
            my $b = new Gtk::HBox 0,5;
            my($r,$prev);
            my $prev_sub = sub { $r = $_[0] };
-           for (@$extra) {
-              my ($label,$value)=@$_;
+           while (@$extra) {
+              my $label = shift @$extra;
+              my $value = shift @$extra;
               my $radio = new Gtk::RadioButton $label;
               $radio->set_group ($prev) if $prev;
               $b->pack_start ($radio,1,0,5);
@@ -436,6 +436,18 @@ sub interact($$$@) {
            push(@setvals,$widget[1]);
            push(@getvals,$widget[2]);
            
+        } elsif($type == PF_FILE) {
+           &new_PF_STRING;
+           my $s = $a;
+           $a = new Gtk::HBox 0,5;
+           $a->add ($s);
+           my $b = new Gtk::Button "Browse";
+           $a->add ($b);
+           my $f = new Gtk::FileSelection $desc;
+           $b->signal_connect (clicked => sub { $f->set_filename ($s->get_text); $f->show_all });
+           $f->ok_button    ->signal_connect (clicked => sub { $f->hide; $s->set_text ($f->get_filename) });
+           $f->cancel_button->signal_connect (clicked => sub { $f->hide });
+           
         } else {
            $label="Unsupported argumenttype $type";
            push(@setvals,sub{});
@@ -537,6 +549,7 @@ sub string2pf($$) {
       || $type==PF_PATTERN
       || $type==PF_BRUSH
       || $type==PF_CUSTOM
+      || $type==PF_FILE
       || $type==PF_RADIO	# for now! #d#
       || $type==PF_GRADIENT) {
       $s;
@@ -653,16 +666,17 @@ sub query {
       Gimp->gimp_install_procedure($function,$blurb,$help,$author,$copyright,$date,
                                    $menupath,$imagetypes,$type,
                                    [map {
-                                      $_->[0]=PARAM_INT32	if $_->[0] == PF_TOGGLE;
-                                      $_->[0]=PARAM_INT32	if $_->[0] == PF_SLIDER;
-                                      $_->[0]=PARAM_INT32	if $_->[0] == PF_SPINNER;
-                                      $_->[0]=PARAM_INT32	if $_->[0] == PF_ADJUSTMENT;
-                                      $_->[0]=PARAM_INT32	if $_->[0] == PF_RADIO;
-                                      $_->[0]=PARAM_STRING	if $_->[0] == PF_FONT;
-                                      $_->[0]=PARAM_STRING	if $_->[0] == PF_BRUSH;
-                                      $_->[0]=PARAM_STRING	if $_->[0] == PF_PATTERN;
-                                      $_->[0]=PARAM_STRING	if $_->[0] == PF_GRADIENT;
-                                      $_->[0]=PARAM_STRING	if $_->[0] == PF_CUSTOM;
+                                      $_->[0]=Gimp::PARAM_INT32		if $_->[0] == PF_TOGGLE;
+                                      $_->[0]=Gimp::PARAM_INT32		if $_->[0] == PF_SLIDER;
+                                      $_->[0]=Gimp::PARAM_INT32		if $_->[0] == PF_SPINNER;
+                                      $_->[0]=Gimp::PARAM_INT32		if $_->[0] == PF_ADJUSTMENT;
+                                      $_->[0]=Gimp::PARAM_INT32		if $_->[0] == PF_RADIO;
+                                      $_->[0]=Gimp::PARAM_STRING	if $_->[0] == PF_FONT;
+                                      $_->[0]=Gimp::PARAM_STRING	if $_->[0] == PF_BRUSH;
+                                      $_->[0]=Gimp::PARAM_STRING	if $_->[0] == PF_PATTERN;
+                                      $_->[0]=Gimp::PARAM_STRING	if $_->[0] == PF_GRADIENT;
+                                      $_->[0]=Gimp::PARAM_STRING	if $_->[0] == PF_CUSTOM;
+                                      $_->[0]=Gimp::PARAM_STRING	if $_->[0] == PF_FILE;
                                       $_;
                                    } @$params],
                                    $results);
@@ -822,11 +836,11 @@ The same as PF_SLIDER, except that this one uses a spinbutton instead of a scale
 
 In addition to a default value, an extra argument describing the various
 options I<must> be provided. That extra argument must be a reference
-to an array filled with ["Option-Name", integer-value] pairs. Gimp::Fu
+to an array filled with C<Option-Name => Option-Value> pairs. Gimp::Fu
 will then generate a horizontal frame with radio buttons, one for each
 alternative. For example:
 
- [PF_RADIO, "direction", "the direction to move to", 5, [["Left",5],["Right",7]]]
+ [PF_RADIO, "direction", "the direction to move to", 5, [Left => 5,  Right => 7]]]
 
 draws two buttons, when the first (the default, "Left") is activated, 5
 will be returned. If the second is activated, 7 is returned.
@@ -864,6 +878,11 @@ While the values can be of any type (as long as it fits into a scalar),
 you should be prepared to get a string when the script is started from the
 commandline or via the PDB.
 
+=item PF_FILE
+
+This represents a file system object. It usually is a file, but can be
+anything (directory, link). It might not even exist at all.
+
 =back
 
 =cut
@@ -881,16 +900,22 @@ sub register($$$$$$$$$;@) {
    @_==0 or die "register called with too many or wrong arguments\n";
    
    for my $p (@$params,@$results) {
-      int($p->[0]) eq $p->[0] or croak "Argument/return value '$p->[1]' has illegal type '$p->[0]'";
+      int($p->[0]) eq $p->[0] or croak "$function: argument/return value '$p->[1]' has illegal type '$p->[0]'";
+      $p->[1]=~/^[0-9a-z_]+$/ or carp "$function: argument name '$p->[1]' contains illegal characters, only 0-9, a-z and _ allowed";
    }
    
    $function="perl_fu_".$function unless $function=~/^(?:perl_fu|extension|plug_in)/ || $function=~s/^\+//;
    
+   Gimp::logger message => "function name contains dashes instead of underscores",
+                function => $function, fatal => 0
+      if $function =~ y/-//;
+
    *$function = sub {
       $run_mode=shift;	# global!
       my(@pre,@defaults,@lastvals,$input_image);
+
       if ($menupath=~/^<Image>\//) {
-         @_ >= 2 or die "<Image> plug-in called without an image and drawable!\n";
+         @_ >= 2 or die "<Image> plug-in called without both image and drawable arguments!\n";
          @pre = (shift,shift);
       } elsif ($menupath=~/^<Toolbox>\//) {
          # valid ;)
@@ -920,7 +945,7 @@ sub register($$$$$$$$$;@) {
          }
       } elsif ($run_mode == &Gimp::RUN_FULLINTERACTIVE) {
          my($res);
-         ($res,@_)=interact($function,$blurb,$help,[@image_params,@{$params}],[@pre,@_]);
+         ($res,@_)=interact($function,$blurb,$help,[@image_params,@$params],[@pre,@_]);
          undef @pre;
          return unless $res;
       } elsif ($run_mode == &Gimp::RUN_NONINTERACTIVE) {
@@ -933,7 +958,8 @@ sub register($$$$$$$$$;@) {
       $input_image = $_[0]   if ref $_[0]   eq "Gimp::Image";
       $input_image = $pre[0] if ref $pre[0] eq "Gimp::Image";
       
-      $Gimp::Data{"$function/_fu_data"}=Dumper([@_]);
+      eval { require Data::Dumper };
+      $Gimp::Data{"$function/_fu_data"}=Data::Dumper::Dumper([@_]) unless $@;
       
       print $function,"(",join(",",(@pre,@_)),")\n" if $Gimp::verbose;
       
@@ -964,6 +990,7 @@ sub register($$$$$$$$$;@) {
       }
       Gimp->displays_flush;
       
+      Gimp::set_trace ($old_trace);
       wantarray ? @imgs : $imgs[0];
    };
    push(@scripts,[$function,$blurb,$help,$author,$copyright,$date,
