@@ -85,13 +85,13 @@ register "seth_spin",
          "<Toolbox>/Xtns/Animation/Seth Spin",
          "*",
          [
-          [PF_DRAWABLE, "Source", "What drawable to spin from?"],
-          [PF_DRAWABLE, "Destination","What drawable to spin to?"],
-		  [PF_INT8, "Frames", "How many frames to use?", 16],
-		  [PF_COLOR, "Background", "What color to use for background if not transparent", [0,0,0]],
-		  [PF_SLIDER, "Perspective", "How much perspective effect to get", 40, [0,255,5]],
-		  [PF_TOGGLE, "Spin Back", "Also spin back?" , 1],
-          [PF_TOGGLE, "Convert Indexed", "Convert to indexed?", 1],
+          [PF_DRAWABLE, "source", "What drawable to spin from?"],
+          [PF_DRAWABLE, "destination","What drawable to spin to?"],
+		  [PF_INT8, "frames", "How many frames to use?", 16],
+		  [PF_COLOR, "background", "What color to use for background if not transparent", [0,0,0]],
+		  [PF_SLIDER, "perspective", "How much perspective effect to get", 40, [0,255,5]],
+		  [PF_TOGGLE, "spin_back", "Also spin back?" , 1],
+          [PF_TOGGLE, "convert_indexed", "Convert to indexed?", 1],
          ],
          [],
          ['gimp-1.1'],
