@@ -12,18 +12,17 @@ use PDL::Core;
 use PDL;
 use Gimp::PDL;
 
-register (
+register
     'view3d',
     'View grayscale drawable in 3D',
     'This script uses PDL::Graphics:TriD to view a grayscale drawable in 3D. You can choose a Cartesian (default) or Polar projection, toggle the drawing of lines, and toggle normal smoothing.',
     'Tom Rathborne', 'GPLv2', '1999-03-11',
     '<Image>/View/3D Surface',
     'GRAY', [
-        [ PF_BOOL, 'Polar', 'Radial view', 0],
-        [ PF_BOOL, 'Lines', 'Draw grid lines', 0],
-        [ PF_BOOL, 'Smooth', 'Smooth surface normals', 1]
+        [ PF_BOOL, 'polar', 'Radial view', 0],
+        [ PF_BOOL, 'lines', 'Draw grid lines', 0],
+        [ PF_BOOL, 'smooth', 'Smooth surface normals', 1]
     ], [],
-
 sub {
     my ($img, $dwb, $polar, $lines, $smooth) = @_;
    
@@ -38,10 +37,8 @@ sub {
     imag3d [ $polar ? 'POLAR2D' : 'SURF2D', $surf ],
            { 'Lines' => $lines, 'Smooth' => $smooth };
 
-    0;
-}
-
-);
+    ();
+};
 
 exit main;
 
